#!/usr/bin/python
'''
LED Matrix
===========
- configure LED matrix via GUI, generate Arduino .ino file
- by AM 2017-2018 (andreas.moeglich@uni-bayreuth.de)
- note: this version ported to Python 3 and PyQt5
'''

from PyQt5 import QtCore, QtGui, QtWidgets
from functools import partial
import re
import sys

try:
  _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
  def _fromUtf8(s):
    return s

try:
  _encoding = QtWidgets.QApplication.UnicodeUTF8
  def _translate(context, text, disambig):
    return QtWidgets.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
  def _translate(context, text, disambig):
    return QtWidgets.QApplication.translate(context, text, disambig)

# a QLineEdit that finally behaves as I want
class QLineEditClick(QtWidgets.QLineEdit):
  def __init__(self, argument = None):
    super(QLineEditClick, self).__init__(argument)
    self._gainedFocus = False
    self._originalMousePressEvent = QtWidgets.QLineEdit.mousePressEvent
    self._originalFocusInEvent = QtWidgets.QLineEdit.focusInEvent

  def focusInEvent(self, event):
    self._originalFocusInEvent(self, event)
    self.selectAll()
    
    # determine how we got focus (0 is via mouse action)
    if(event.reason() == 0):
      self._gainedFocus = True
    
  def mousePressEvent(self, event):
    self._originalMousePressEvent(self, event)
    if(self._gainedFocus):
      self.selectAll()
      self._gainedFocus = False

class myLineEdit(QLineEditClick):
  def __init__(self,parent,currled=0,field=0,mainUI=None):
    super(myLineEdit,self).__init__(parent)
   # QtGui.QLineEdit.__init__(self,parent)
    self.currled=currled
    self.field=field
    self.mainUI=mainUI

  # capture arrow keys
  def event(self, event):
    if (event.type()==QtCore.QEvent.KeyPress) and (event.key()==QtCore.Qt.Key_Up):
      if (self.field>3):
        self.mainUI.leds[self.currled]['qedit'][self.field-4].setFocus()
        self.mainUI.leds[self.currled]['qedit'][self.field-4].selectAll()
        self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled]['qedit'][self.field-4])
      elif(self.currled):
        if(self.field):
          # extra check whether field is enabled
          if (self.mainUI.leds[self.currled-1]['qedit'][self.field+4].isEnabled()):
            self.mainUI.leds[self.currled-1]['qedit'][self.field+4].setFocus()
            self.mainUI.leds[self.currled-1]['qedit'][self.field+4].selectAll()
            self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled-1]['qedit'][self.field+4])
          else:
            self.mainUI.leds[self.currled-1]['qedit'][self.field].setFocus()
            self.mainUI.leds[self.currled-1]['qedit'][self.field].selectAll()
            self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled-1]['qedit'][self.field])
        else:
          self.mainUI.leds[self.currled-1]['qedit'][8].setFocus()
          self.mainUI.leds[self.currled-1]['qedit'][8].selectAll()
          self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled-1]['qedit'][8])
      return True
    elif (event.type()==QtCore.QEvent.KeyPress) and (event.key()==QtCore.Qt.Key_Down):
      if (self.field<5):
        # extra check whether field is enabled
        if (self.mainUI.leds[self.currled]['qedit'][self.field+4].isEnabled()):
          self.mainUI.leds[self.currled]['qedit'][self.field+4].setFocus()
          self.mainUI.leds[self.currled]['qedit'][self.field+4].selectAll()
          self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled]['qedit'][self.field+4])
        elif(self.currled<self.mainUI.LED_NUMBER-1):
          self.mainUI.leds[self.currled+1]['qedit'][self.field].setFocus()
          self.mainUI.leds[self.currled+1]['qedit'][self.field].selectAll()
          self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled+1]['qedit'][self.field])
      elif(self.currled<self.mainUI.LED_NUMBER-1):
        if(self.field != 8):
          self.mainUI.leds[self.currled+1]['qedit'][self.field-4].setFocus()
          self.mainUI.leds[self.currled+1]['qedit'][self.field-4].selectAll()
          self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled+1]['qedit'][self.field-4])
        else:
          self.mainUI.leds[self.currled+1]['qedit'][0].setFocus()
          self.mainUI.leds[self.currled+1]['qedit'][0].selectAll()
          self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled+1]['qedit'][0])
      return True

    return QtWidgets.QLineEdit.event(self, event)
    
class PaintArea(QtWidgets.QWidget):
  def __init__(self,numled=64,mainUI=None):
    super(PaintArea,self).__init__()
    self.mainUI=mainUI
    self.CIRCLE_RADIUS=int(40*(1.0*mainUI.DPI_SCALING/2))
    self.CIRCLE_BUFFER=int(5*(1.0*mainUI.DPI_SCALING/2))
    self.ledcolor=[[255,255,255] for i in range(numled)]
    self.show()

  def paintEvent(self, event):
    self.qp = QtGui.QPainter(self)
    # don't need explicit begin() call as constructor does this already
    #self.qp.begin(self)

    self.mybrush=QtGui.QBrush(QtCore.Qt.red)
    self.mybrush.setColor(QtCore.Qt.black)
    self.qp.setBrush(self.mybrush)
    self.mypen=QtGui.QPen()
    self.mypen.setWidth(1.5)
    self.mypen.setColor(QtCore.Qt.black)
    self.qp.setPen(self.mypen)
    self.qp.drawRoundedRect(0, 0, 8*self.CIRCLE_RADIUS+9*self.CIRCLE_BUFFER,\
      8*self.CIRCLE_RADIUS+9*self.CIRCLE_BUFFER, 10.0, 10.0)
    
    self.mypen.setWidth(0.5)
    self.mypen.setColor(QtCore.Qt.white)
    self.qp.setPen(self.mypen)
    
    for row in range(8):
      for col in range(8):
        self.mybrush.setColor(QtGui.QColor(*self.ledcolor[row+col*8]))
        self.qp.setBrush(self.mybrush)
        self.qp.drawEllipse(self.CIRCLE_BUFFER+row*(self.CIRCLE_RADIUS+self.CIRCLE_BUFFER),\
          self.CIRCLE_BUFFER+col*(self.CIRCLE_RADIUS+self.CIRCLE_BUFFER),self.CIRCLE_RADIUS,\
          self.CIRCLE_RADIUS)

    self.qp.end()
    
  def update_color(self,number=0,rgb=[0,0,0]):
    self.ledcolor[number]=rgb
    self.update()
    
  def mousePressEvent(self, QMouseEvent):
    # get x,y coords of click
    coordx=QMouseEvent.pos().x()-self.CIRCLE_BUFFER
    coordy=QMouseEvent.pos().y()-self.CIRCLE_BUFFER
    
    # calculate which LED was clicked
    row=int(coordy/(self.CIRCLE_RADIUS+self.CIRCLE_BUFFER))
    col=int(coordx/(self.CIRCLE_RADIUS+self.CIRCLE_BUFFER))
    if((row<8) and (col<8)):
      clickedLED=col+row*8
      self.mainUI.select_LED(clickedLED)

class Ui_MainWindow(object):
  def setupUi(self, MainWindow,DPI_SCALING=1):
    self.DPI_SCALING=DPI_SCALING
    # set up main window
    MainWindow.setObjectName(_fromUtf8("MainWindow"))
    self.WINDOW_WIDTH=int(800*(1.0*self.DPI_SCALING/2))
    self.WINDOW_HEIGHT=int(600*(1.0*self.DPI_SCALING/2))
    MainWindow.resize(self.WINDOW_WIDTH,self.WINDOW_HEIGHT)
    
    self.centralwidget = QtWidgets.QWidget(MainWindow)
    self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
    
    self.horizontalLayout1=QtWidgets.QHBoxLayout(self.centralwidget)
    self.verticalLayout1 = QtWidgets.QVBoxLayout()
    self.horizontalLayout1.addLayout(self.verticalLayout1)
    
    # build left panel
    self.scrollArea = QtWidgets.QScrollArea(self.centralwidget)
    self.scrollArea.setMinimumSize(QtCore.QSize(self.WINDOW_WIDTH,self.WINDOW_HEIGHT))
    self.scrollArea.setMaximumSize(QtCore.QSize(self.WINDOW_WIDTH,16777215))
    self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
    self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
    self.scrollArea.setWidgetResizable(True)
    
    self.verticalLayout1.addWidget(self.scrollArea)

    self.ledwidget = QtWidgets.QWidget(self.scrollArea)
    self.ledwidget.setGeometry(QtCore.QRect(0, 0,int(780*(1.0*self.DPI_SCALING/2)),\
      int(800*(1.0*self.DPI_SCALING/2))))
    self.ledwidget.setObjectName(_fromUtf8("ledwidget"))
    
    self.verticalLayout = QtWidgets.QVBoxLayout(self.ledwidget)
    self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
    self.verticalLayout.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)

    # set up validators
    self.valid_color=QtGui.QIntValidator(0,255)
    self.valid_time=QtGui.QIntValidator()
    self.valid_time.setBottom(0)
    
    # set up ressource
    # size definitions for DPI scaling
    xoff=int(10*(1.0*self.DPI_SCALING/2))
    yoff=int(10*(1.0*self.DPI_SCALING/2))
    # for qlabels
    self.CONTAINER_WIDTH=int(111*(1.0*self.DPI_SCALING/2))
    self.CONTAINER_HEIGHT=int(41*(1.0*self.DPI_SCALING/2))
    self.LABEL_HEIGHT=int(27*(1.0*self.DPI_SCALING/2))
    self.LABEL_WIDTH1=int(91*(1.0*self.DPI_SCALING/2))
    self.LABEL_WIDTH2=int(31*(1.0*self.DPI_SCALING/2))
    self.XOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (0,110,230,350,520)]
    self.YOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (60,110,160)]
    # for radio buttons
    self.RADIO_CONTAINER_WIDTH=int(270*(1.0*self.DPI_SCALING/2))
    self.RADIO_CONTAINER_HEIGHT=int(50*(1.0*self.DPI_SCALING/2))
    self.RADIO_WIDTH=int(120*(1.0*self.DPI_SCALING/2))
    self.RADIO_HEIGHT=int(31*(1.0*self.DPI_SCALING/2))
    self.RADIO_XOFFSET=int(135*(1.0*self.DPI_SCALING/2))
    # for copy elements
    self.COPY_XOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (400,510)]
    self.COPY_CHECK_WIDTH=int(50*(1.0*self.DPI_SCALING/2))
    self.COPY_CHECK_HEIGHT=self.COPY_CHECK_WIDTH
    self.COPY_WIDTH=int(100*(1.0*self.DPI_SCALING/2))
    self.COPY_HEIGHT=int(40*(1.0*self.DPI_SCALING/2))
    # for qlineedits
    self.QLINEEDIT_XOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (140,260,380,610)]
    self.QLINEEDIT_YOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (50,100,150)]
    self.QLINEEDIT_WIDTH1=int(71*(1.0*self.DPI_SCALING/2))
    self.QLINEEDIT_WIDTH2=int(111*(1.0*self.DPI_SCALING/2))
    self.QLINEEDIT_HEIGHT=int(41*(1.0*self.DPI_SCALING/2))
    # for colored boxes
    self.CBOX_XOFFSET=int(470*(1.0*self.DPI_SCALING/2))
    self.CBOX_YOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (50,100)]
    self.CBOX_WIDTH=int(40*(1.0*self.DPI_SCALING/2))
    self.CBOX_HEIGHT=self.CBOX_WIDTH#
    
    # set up entries
    self.LED_NUMBER=64
    self.leds=[]
    for currled in range(self.LED_NUMBER):
      # initialize and set offset
      self.leds.append({})
      
      self.leds[-1]['container']=QtWidgets.QGroupBox(self.ledwidget)
      self.leds[-1]['container'].setMinimumSize(QtCore.QSize(int(740*(1.0*self.DPI_SCALING/2)),\
        int(210*(1.0*self.DPI_SCALING/2))))
      self.leds[-1]['container'].setMaximumSize(QtCore.QSize(int(740*(1.0*self.DPI_SCALING/2)),\
        int(210*(1.0*self.DPI_SCALING/2))))
      
      # build elements    
      # set up labels
      self.leds[-1]['qlabels']=[]
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff,yoff,self.CONTAINER_WIDTH,self.CONTAINER_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:12pt;\">LED "+str(currled)+"</span></p></body></html>", None))
      
      # 1st row
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[0],yoff+self.YOFFSET[0],\
        self.LABEL_WIDTH1,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Color 1</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[1],yoff+self.YOFFSET[0],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">R</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[2],yoff+self.YOFFSET[0],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">G</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[3],yoff+self.YOFFSET[0],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">B</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[4],yoff+self.YOFFSET[0],\
        self.LABEL_WIDTH1,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Time 1</span></p></body></html>", None))

      # 2nd row
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[0],yoff+self.YOFFSET[1],\
        self.LABEL_WIDTH1,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Color 2</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[1],yoff+self.YOFFSET[1],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">R</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[2],yoff+self.YOFFSET[1],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">G</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[3],yoff+self.YOFFSET[1],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">B</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[4],yoff+self.YOFFSET[1],\
        self.LABEL_WIDTH1,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Time 2</span></p></body></html>", None))

      
      # set up radio buttons
      self.leds[-1]['qradio']=[]
      self.leds[-1]['qradio'].append(QtWidgets.QGroupBox(self.leds[-1]['container']))
      self.leds[-1]['qradio'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[1],yoff,self.RADIO_CONTAINER_WIDTH,self.RADIO_CONTAINER_HEIGHT))
      
      self.leds[-1]['qradio'].append(QtWidgets.QRadioButton(self.leds[-1]['qradio'][0]))
      self.leds[-1]['qradio'][-1].setGeometry(QtCore.QRect(xoff/2, yoff/2, self.RADIO_WIDTH, self.RADIO_HEIGHT))
      self.leds[-1]['qradio'][-1].setChecked(True)
      self.leds[-1]['qradio'][-1].setText(_translate("MainWindow", "box car", None))
      self.leds[-1]['qradio'].append(QtWidgets.QRadioButton(self.leds[-1]['qradio'][0]))
      self.leds[-1]['qradio'][-1].setGeometry(QtCore.QRect(self.RADIO_XOFFSET, yoff/2, self.RADIO_WIDTH, self.RADIO_HEIGHT))
      self.leds[-1]['qradio'][-1].setChecked(False)
      self.leds[-1]['qradio'][-1].setText(_translate("MainWindow", "sinusoid", None))
      
      # set up check box and copy button
      self.leds[-1]['check']=[]
      self.leds[-1]['check'].append(QtWidgets.QCheckBox(self.leds[-1]['container']))
      self.leds[-1]['check'][0].setGeometry(QtCore.QRect(xoff+self.COPY_XOFFSET[1],yoff,\
        self.COPY_CHECK_WIDTH,self.COPY_CHECK_HEIGHT))
      self.leds[-1]['check'][0].setChecked(False)
      
      self.leds[-1]['check'].append(QtWidgets.QPushButton(self.leds[-1]['container']))
      self.leds[-1]['check'][1].setGeometry(QtCore.QRect(xoff+self.COPY_XOFFSET[0],yoff+5,\
        self.COPY_WIDTH,self.COPY_HEIGHT))
      self.leds[-1]['check'][1].setText('Copy')

      # set up qedits
      # 1st row
      self.leds[-1]['qedit']=[]
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,0,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[0],\
        yoff+self.QLINEEDIT_YOFFSET[0],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,1,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[1],\
        yoff+self.QLINEEDIT_YOFFSET[0],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,2,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[2],\
        yoff+self.QLINEEDIT_YOFFSET[0],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,3,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[3],\
        yoff+self.QLINEEDIT_YOFFSET[0],self.QLINEEDIT_WIDTH2,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(10)
      self.leds[-1]['qedit'][-1].setText(str(5000))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_time)

      # 2nd row
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,4,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[0],\
        yoff+self.QLINEEDIT_YOFFSET[1],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,5,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[1],\
        yoff+self.QLINEEDIT_YOFFSET[1],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,6,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[2],\
        yoff+self.QLINEEDIT_YOFFSET[1],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,7,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[3],\
        yoff+self.QLINEEDIT_YOFFSET[1],self.QLINEEDIT_WIDTH2,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(10)
      self.leds[-1]['qedit'][-1].setText(str(1000))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_time)

      # set up color boxes
      # 1st row
      self.leds[-1]['cbox']=[]
      self.leds[-1]['cbox'].append(QtWidgets.QGroupBox(self.leds[-1]['container']))
      self.leds[-1]['cbox'][-1].setGeometry(QtCore.QRect(xoff+self.CBOX_XOFFSET,\
        yoff+self.CBOX_YOFFSET[0],self.CBOX_WIDTH,self.CBOX_HEIGHT))
      self.leds[-1]['cbox'][-1].setAutoFillBackground(False)
      self.leds[-1]['cbox'][-1].setStyleSheet(_fromUtf8("background-color: rgb(21, 236, 255);"))

      # 2nd row
      self.leds[-1]['cbox'].append(QtWidgets.QGroupBox(self.leds[-1]['container']))
      self.leds[-1]['cbox'][-1].setGeometry(QtCore.QRect(xoff+self.CBOX_XOFFSET,\
        yoff+self.CBOX_YOFFSET[1],self.CBOX_WIDTH,self.CBOX_HEIGHT))
      self.leds[-1]['cbox'][-1].setAutoFillBackground(False)
      self.leds[-1]['cbox'][-1].setStyleSheet(_fromUtf8("background-color: rgb(21, 236, 255);"))

      # 3rd row
      self.leds[-1]['qlabels'].append(QtWidgets.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[0],yoff+self.YOFFSET[2],\
        2*self.LABEL_WIDTH1,self.LABEL_HEIGHT + int(5*(1.0*self.DPI_SCALING/2))))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Init Delay</span></p></body></html>", None))

      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,8,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[0],\
        yoff+self.QLINEEDIT_YOFFSET[2],self.QLINEEDIT_WIDTH2,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(10)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_time)

      self.verticalLayout.addWidget(self.leds[-1]['container'])
      
    # other stuff
    self.scrollArea.setWidget(self.ledwidget)

    # build right panel
    self.verticalLayout2 = QtWidgets.QVBoxLayout()
    self.horizontalLayout1.addLayout(self.verticalLayout2)
    self.verticalLayout2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
    
    self.BUTTON_WIDTH=int(200*(1.0*self.DPI_SCALING/2))
    self.BUTTON_HEIGHT=int(50*(1.0*self.DPI_SCALING/2))
    self.saveButton=QtWidgets.QPushButton()
    self.saveButton.setGeometry(QtCore.QRect(0,0,self.BUTTON_WIDTH,self.BUTTON_HEIGHT))
    self.saveButton.setText('Save')
    self.saveButton.clicked.connect(self.save)
    self.verticalLayout2.addWidget(self.saveButton)
    
    self.loadButton=QtWidgets.QPushButton()
    self.loadButton.setGeometry(QtCore.QRect(0,0,self.BUTTON_WIDTH,self.BUTTON_HEIGHT))
    self.loadButton.setText('Load')
    self.loadButton.clicked.connect(self.load)
    self.verticalLayout2.addWidget(self.loadButton)
    
    self.exportButton=QtWidgets.QPushButton()
    self.exportButton.setGeometry(QtCore.QRect(0,0,self.BUTTON_WIDTH,self.BUTTON_HEIGHT))
    self.exportButton.setText('Export')
    self.exportButton.clicked.connect(self.export)
    self.verticalLayout2.addWidget(self.exportButton)
    
    self.pushButton = QtWidgets.QPushButton(self.centralwidget)
    self.pushButton.setGeometry(QtCore.QRect())
    self.pushButton.setText('Quit')
    self.pushButton.clicked.connect(MainWindow.close)
    self.verticalLayout2.addWidget(self.pushButton)

    # set up RTC check box and button
    self.layoutRTC=QtWidgets.QHBoxLayout()
    self.layoutRTC.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
    self.verticalLayout2.addLayout(self.layoutRTC)
    
    self.checkRTC=QtWidgets.QCheckBox()
    self.checkRTC.setGeometry(QtCore.QRect(xoff,yoff,\
      self.COPY_CHECK_WIDTH,self.COPY_CHECK_HEIGHT))
    self.checkRTC.setChecked(False)
    self.layoutRTC.addWidget(self.checkRTC)
    
    self.labelRTC=QtWidgets.QLabel()
    self.labelRTC.setGeometry(QtCore.QRect(xoff,yoff,\
      self.LABEL_WIDTH1,self.LABEL_HEIGHT))
    self.labelRTC.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Include RTC?</span></p></body></html>", None))
    self.layoutRTC.addWidget(self.labelRTC)
    
    self.preview=QtWidgets.QLabel()
    self.preview.setGeometry(QtCore.QRect(0,0,int(390*(1.0*self.DPI_SCALING/2)),int(100*(1.0*self.DPI_SCALING/2))))
    self.preview.setText(_translate("MainWindow", "<html><head/><body>\
      <p><span style=\" font-size:10pt;color:#777777;font-style:normal;\">Matrix Preview</span></p>\
      </body></html>", None))
    self.verticalLayout2.addWidget(self.preview)

    self.PAINTAREA_SIZE=int(370*(1.0*self.DPI_SCALING/2))
    self.qpArea2=PaintArea(self.LED_NUMBER,self)
    self.qpArea2.setMinimumSize(QtCore.QSize(self.PAINTAREA_SIZE,self.PAINTAREA_SIZE))
    self.qpArea2.setMaximumSize(QtCore.QSize(self.PAINTAREA_SIZE,self.PAINTAREA_SIZE))
    self.qpArea2.setCursor(QtCore.Qt.PointingHandCursor)
    self.verticalLayout2.addWidget(self.qpArea2)
    
    self.about=QtWidgets.QLabel()
    self.about.setGeometry(QtCore.QRect(0,0,int(390*(1.0*self.DPI_SCALING/2)),int(100*(1.0*self.DPI_SCALING/2))))
    self.about.setText(_translate("MainWindow", "<html><head/><body>\
      <p><span style=\" font-size:12pt;\">LED Matrix v0.21</span></p>\
      <p><span style=\" font-size:10pt;\">by M&ouml;glich lab 2017/18</span></p>\
      <p><span style=\" font-size:10pt;\">University of Bayreuth</span></p></body></html>", None))
    self.verticalLayout2.addWidget(self.about)

    # set up connectors
    for currled in range(self.LED_NUMBER):
      # connect events
      # radio buttons => disable Time2
      self.leds[currled]['qradio'][-2].toggled.connect(partial(self.disable_entry,self.leds[currled]['qedit'][-2],True))
      self.leds[currled]['qradio'][-1].toggled.connect(partial(self.disable_entry,self.leds[currled]['qedit'][-2],False))

      # edit fields => update color
      for i in range(3):
        self.leds[currled]['qedit'][i].textChanged.connect(partial(self.update_color,self.leds[currled]['cbox'][0],currled,1))
      self.update_color(self.leds[currled]['cbox'][0],currled,1)
      for i in range(3):
        self.leds[currled]['qedit'][i+4].textChanged.connect(partial(self.update_color,self.leds[currled]['cbox'][1],currled,2))
      self.update_color(self.leds[currled]['cbox'][1],currled,2)
      
      # copy button
      self.leds[currled]['check'][1].clicked.connect(partial(self.copy,currled))

    # main window
    MainWindow.setCentralWidget(self.centralwidget)
    MainWindow.setWindowTitle(_translate("MainWindow", "LED Matrix", None))
    self.CENTRAL_WIDTH=int(1220*(1.0*self.DPI_SCALING/2))
    self.CENTRAL_HEIGHT=int(850*(1.0*self.DPI_SCALING/2))
    self.centralwidget.setMinimumSize(QtCore.QSize(self.CENTRAL_WIDTH,self.CENTRAL_HEIGHT))
    self.centralwidget.setMaximumSize(QtCore.QSize(self.CENTRAL_WIDTH,16777215))
    self.leds[0]['qedit'][0].setFocus()
    self.leds[0]['qedit'][0].selectAll()

  def disable_entry(self,target=None,value=False):
    if (target):
      target.setEnabled(value)
    
  def update_color(self,target=None,currled=0,color=1):
    # update cbox entry
    rgb=[self.leds[currled]['qedit'][i+(color-1)*4].text() for i in range(3)]
    rgb=[str(i) for i in rgb]
    rgb=[i if (len(i)) else '0' for i in rgb]
    colstr=','.join(rgb)
    target.setStyleSheet(_fromUtf8("background-color: rgb("+colstr+");"))
    
    # update LED matrix cartoon
    if (color==1):
      self.qpArea2.update_color(currled,[int(i) for i in rgb])
      
  def save(self):
    # does what it says
    filename,filter_=QtWidgets.QFileDialog.getSaveFileName(None,filter='Config Files (*.cfg);;Text Files (*.txt);;All files (*.*)')
    filename=str(filename)
    
    # now save parameters to file
    try:
      writehandle=open(filename,'w')
      
      if(writehandle):
        # write general stuff
        writehandle.write('<no. LEDs>'+str(self.LED_NUMBER)+'</no. LEDs>\n')
        
        for i in range(self.LED_NUMBER):
          writehandle.write('<LED '+str(i)+'>\n')
          writehandle.write('\t<mode>'+str(int(self.leds[i]['qradio'][2].isChecked()))+'</mode>\n')
          col1=[str(self.leds[i]['qedit'][j].text()) for j in range(3)]
          col1=','.join([j if(len(j)) else '0' for j in col1])
          writehandle.write('\t<color1>'+col1+'</color1>\n')
          col2=[str(self.leds[i]['qedit'][4+j].text()) for j in range(3)]
          col2=','.join([j if(len(j)) else '0' for j in col2])
          writehandle.write('\t<color2>'+col2+'</color2>\n')
          times=[str(self.leds[i]['qedit'][j].text()) for j in (3,7,8)]
          times=[j if (len(j)) else '0' for j in times]
          writehandle.write('\t<time1>'+times[0]+'</time1>\n')
          writehandle.write('\t<time2>'+times[1]+'</time2>\n')
          writehandle.write('\t<time3>'+times[2]+'</time3>\n')
          writehandle.write('</LED '+str(i)+'>\n')
        
        writehandle.close()
    except:
      pass

  def load(self):
    # does what it says
    filename,filter_=QtWidgets.QFileDialog.getOpenFileName(None,filter='Config Files (*.cfg);;Text Files (*.txt);;All files (*.*)')
    filename=str(filename)
    
    # now read parameters from file
    try:
      readhandle=open(filename,'r')
      
      if(readhandle):
        # parse entire file
        parsed=readhandle.readlines()
        readhandle.close()
        parsed=''.join(parsed)
        parsed=re.sub('\s','',parsed)
        
        # search for number of LEDs
        mt=re.match('.*<no\.LEDs>(.+)</no\.LEDs>',parsed)
        if (mt):
          noleds=int(mt.group(1))
          for i in range(noleds):
            matchstring='.*<LED'+str(i)+'>(.+)</LED'+str(i)+'>'
            mt=re.match(matchstring,parsed)#,re.DOTALL)
            if (mt):
              parse2=mt.group(1)
              
              # read out mode
              mt=re.match('.*<mode>(.+)</mode>',parse2)
              if (mt):
                if (bool(int(mt.group(1)))):
                  self.leds[i]['qradio'][2].setChecked(True)
                else:
                  self.leds[i]['qradio'][1].setChecked(True)
  
              # read out color1
              mt=re.match('.*<color1>(.+)</color1>',parse2)
              if (mt):
                self.rgb=mt.group(1).split(',')
                for j in range(3):
                  self.leds[i]['qedit'][j].setText(self.rgb[j])
  
              # read out color2
              mt=re.match('.*<color2>(.+)</color2>',parse2)
              if (mt):
                self.rgb=mt.group(1).split(',')
                for j in range(3):
                  self.leds[i]['qedit'][j+4].setText(self.rgb[j])
  
              # read out time1
              mt=re.match('.*<time1>(.+)</time1>',parse2)
              if (mt):
                self.leds[i]['qedit'][3].setText(mt.group(1))
  
              # read out time2
              mt=re.match('.*<time2>(.+)</time2>',parse2)
              if (mt):
                self.leds[i]['qedit'][7].setText(mt.group(1))
                
              # read out time3
              mt=re.match('.*<time3>(.+)</time3>',parse2)
              if (mt):
                self.leds[i]['qedit'][8].setText(mt.group(1))
    except:
      pass
              
  def copy(self,source=0):
    # copies information from clicked entry to all other entries that are checked
    # retrieve values from source LED
    radio=self.leds[source]['qradio'][1].isChecked()
    color1=[str(self.leds[source]['qedit'][j].text()) for j in range(3)]
    color2=[str(self.leds[source]['qedit'][j+4].text()) for j in range(3)]
    time1=str(self.leds[source]['qedit'][3].text())
    time2=str(self.leds[source]['qedit'][7].text())
    time3=str(self.leds[source]['qedit'][8].text())
    
    # cycle over all entries
    for i in range(self.LED_NUMBER):
      if (self.leds[i]['check'][0].isChecked()):
        # copy mode
        if (radio):
          self.leds[i]['qradio'][1].setChecked(True)
        else:
          self.leds[i]['qradio'][2].setChecked(True)

        # copy color1
        for j in range(3):
          self.leds[i]['qedit'][j].setText(color1[j])

        # copy color2
        for j in range(3):
          self.leds[i]['qedit'][j+4].setText(color2[j])

        # copy time1
        self.leds[i]['qedit'][3].setText(time1)

        # copy time2
        self.leds[i]['qedit'][7].setText(time2)
        
        # copy time3
        self.leds[i]['qedit'][8].setText(time3)
        
  def select_LED(self,currled=0):
    #print currled
    self.leds[currled]['qedit'][0].setFocus()
    self.leds[currled]['qedit'][0].selectAll()
    self.scrollArea.ensureWidgetVisible(self.leds[currled]['qedit'][0])

  def export(self):
    # exports LED configuration to ino file
    filename,filter_=QtWidgets.QFileDialog.getSaveFileName(None,filter='Arduino Sketch (*.ino);;All files (*.*)')
    filename=str(filename)

    # source code of Arduino File
    arduino=\
'''// define whether RTC is to be used or not
#define USE_RTC <<<RTC_SETTING>>>

// includes for RTC
#if USE_RTC
  #include <Wire.h>
  #include "RTClib.h"
#endif

/*
class PixelTimer
which is an object that should make it easy
to programm a large number of neo pixels to
turn on/off at various intervals unrelated
to each other

Florian Richter, Moeglich Lab HU Berlin
florian.richter.1@hu-berlin.de, feb 2014

extended by the incredible AM
andreas.moeglich@uni-bayreuth.de
*/

#define  num_neopixels 64
#ifndef PI
	#define PI 3.14159265358979323846
#endif

// setup for Colorduino matrix
/*****************************
define the IO
*****************************/
#define RST_BIT 0x04
#define LAT_BIT 0x02
#define SLB_BIT 0x01
#define SCL_BIT 0x40
#define SDA_BIT 0x80

#define RST PORTC
#define LAT PORTC
#define SLB PORTC
#define SDA PORTD
#define SCL PORTD

#define open_line0  {PORTB=0x01;}
#define open_line1  {PORTB=0x02;}
#define open_line2  {PORTB=0x04;}
#define open_line3  {PORTB=0x08;}
#define open_line4  {PORTB=0x10;}
#define open_line5  {PORTB=0x20;}
#define open_line6  {PORTD=0x08;}
#define open_line7  {PORTD=0x10;}
#define close_all_line  {PORTD=0x00;PORTB=0x00;}

unsigned char dots[2][8][8][3] = {0};
//dots matrix
//[2]:Page:one for display, one for receive data
//[8]:Row:8 row in LED plane
//[8]:Column:8 column in one row
//[3]:Color:RGB data: 0 for Red; 1 for green, 2 for Blue
unsigned char Gamma_Value[3] = {63,63,63};
//Gamma correctly value, every LED plane is different.value range is 0~63
//[3]:RGB data, 0 for Red; 1 for green, 2 for Blue
unsigned char Page_Index = 0; // the index of buffer
unsigned char Page_value = 0;

unsigned char row = 0;//the value of row in LED plane, from 0~7
unsigned char column = 0;//the value of every row, from 0~7
unsigned char color = 0;//the value of every dots, 0 is Red, 1 is Green, 2 is Blue

unsigned char line = 0;

/**************************************************
all parts inition functions zone
***************************************************/
void _IO_Init()
{
  DDRD = 0xff; // set all pins direction of PortD
  DDRC = 0xff; // set all pins direction of PortC
  DDRB = 0xff; // set all pins direction of PortB
  
  PORTD = 0x00; // set all pins output is low of PortD
  PORTC = 0x00; // set all pins output is low of PortC
  PORTB = 0x00; // set all pins output is low of PortB
}

void _LED_Init()
{
  LED_RST(1);
  LED_Delay(1);
  LED_RST(0);
  LED_Delay(1);
  LED_RST(1);
  LED_Delay(1);
  
  SetGamma();
  line = 0;
  
}

void _TC2_Init()
{
  TCCR2A |= (1 << WGM21) | (1 << WGM20);   
  TCCR2B |= ((1<<CS22)|(1<<CS20));   // by clk/64
  TCCR2B &= ~((1<<CS21));   // by clk/64
  TCCR2A &= ~((1<<WGM21) | (1<<WGM20));   // Use normal mode
  ASSR |= (1<<AS2);       // Use internal clock - external clock not used in Arduino
  TIMSK2 |= (1<<TOIE2) | (0<<OCIE2B);   //Timer2 Overflow Interrupt Enable
  TCNT2 = 0xff;
  sei();
}

/****************************************************
the timer2 operate functions zone
****************************************************/
ISR(TIMER2_OVF_vect)          //Timer2  Service 
{ 
  cli();  
  TCNT2 = 0x64;      //flash a led matrix frequency is 100.3Hz,period is 9.97ms
  //TCNT2 = 0x63;      //flash a led matrix frequency is 99.66Hz,period is 10.034ms   
    if(line > 7) line = 0;    
    close_all_line;  
    run(line);
    open_line(line);
    line++;
    sei();
}

/****************************************************
the LED Hardware operate functions zone
****************************************************/
void LED_SDA(unsigned char temp)
{
  if (temp) 
    SDA|=SDA_BIT;
  else
    SDA&=~SDA_BIT;
}

void LED_SCL(unsigned char temp)
{
  if (temp) 
    SCL|=SCL_BIT;
  else
    SCL&=~SCL_BIT;
}

void LED_RST(unsigned char temp)
{
  if (temp) 
    RST|=RST_BIT;
  else
    RST&=~RST_BIT;
}

void LED_LAT(unsigned char temp)
{
  if (temp) 
    LAT|=LAT_BIT;
  else
    LAT&=~LAT_BIT;
}

void LED_SLB(unsigned char temp)
{
  if (temp) 
    SLB|=SLB_BIT;
  else
    SLB&=~SLB_BIT;
}

/***************************************************
the LED datas operate functions zone
***************************************************/
void SetGamma()
{
  unsigned char i = 0;
  unsigned char j = 0;
  unsigned char k = 0;
  unsigned char temp = 0;
  LED_LAT(0);
  LED_SLB(0);
  for(k=0;k<8;k++)
    for(i = 3;i > 0 ;i--)
    {
      temp = Gamma_Value[i-1]<<2;
      for(j = 0;j<6;j++)
      {
        if(temp &0x80)
          LED_SDA(1);
        else
          LED_SDA(0);
        
        temp =temp << 1;
        LED_SCL(0);
        LED_SCL(1);
    }
  }
  LED_SLB(1);
}

void run(unsigned char k)
{
  unsigned char i = 0;
  unsigned char j = 0;
  unsigned char p = 0;
  unsigned char temp = 0;
  LED_SLB(1);
  LED_LAT(0);
  for(i = 0;i<8;i++)
  {
    for(j=0;j<3;j++)
    {
      temp = dots[Page_Index][k][i][2-j];
      for(p=0;p<8;p++)
      {
         if(temp & 0x80)
           LED_SDA(1);
         else
           LED_SDA(0);
           
         temp = temp<<1;  
         LED_SCL(0);
         LED_SCL(1);
       }
     }
  }
  LED_LAT(1);
  LED_LAT(0);
}

void open_line(unsigned char x)
{
  switch (x)
  {  
    case 0 :open_line0;
            break;
    case 1 :open_line1;
            break;
    case 2 :open_line2;
            break;
    case 3 :open_line3;
            break;
    case 4 :open_line4;
            break;
    case 5 :open_line5;
            break;
    case 6 :open_line6;
            break;
    case 7 :open_line7;
            break;
    default: close_all_line;
            break;
  }
}
  
/******************************************
the other operate functions zone
******************************************/
void LED_Delay(unsigned char i)
{
  unsigned int y;
  y = i * 10;
  while(y--);
}

void component_colors_from_uint32t(
  uint32_t color,
  uint8_t & red_val,
  uint8_t & green_val,
  uint8_t & blue_val
 )
{
  red_val = (uint8_t)(color >> 16),
  green_val = (uint8_t)(color >>  8),
  blue_val = (uint8_t)color;
}

//
/**************************************
Class PixelTimer
 **************************************/

class PixelTimer {

public:
  PixelTimer();
  
  PixelTimer( 
    uint8_t which_pixel,
    uint8_t intensity_r,
    uint8_t intensity_g,
    uint8_t intensity_b,
    uint8_t intensity_r2,
    uint8_t intensity_g2,
    uint8_t intensity_b2,
    uint8_t mode,
    unsigned long param1, 
    unsigned long param2,
    unsigned long param3
  );

  ~PixelTimer();
  
  void set_values(
    uint8_t which_pixel,
    uint8_t intensity_r,
    uint8_t intensity_g,
    uint8_t intensity_b,
    uint8_t intensity_r2,
    uint8_t intensity_g2,
    uint8_t intensity_b2,
    uint8_t mode,
    unsigned long param1, 
    unsigned long param2, 
    unsigned long param3
  );
  
  void process_time(unsigned long current_time);
  void init_on();

private:
  uint8_t my_pixel_;
  uint8_t intensity_r_;
  uint8_t intensity_g_;
  uint8_t intensity_b_;
  uint8_t intensity_r2_;
  uint8_t intensity_g2_;
  uint8_t intensity_b2_;
  uint8_t mode_;
  unsigned long param1_, param2_, param3_;

  void boxcar (unsigned long current_time);
  void sine (unsigned long current_time);
  void setPixelColor(uint8_t red, uint8_t green, uint8_t blue);
};

PixelTimer::PixelTimer() : my_pixel_(num_neopixels +1), intensity_r_(0), intensity_g_(0), intensity_b_(0), intensity_r2_(0), intensity_g2_(0), intensity_b2_(0), mode_(0), param1_(0), param2_(0), param3_(0)
{}

PixelTimer::PixelTimer(
  uint8_t which_pixel,
  uint8_t intensity_r,
  uint8_t intensity_g,
  uint8_t intensity_b,
  uint8_t intensity_r2,
  uint8_t intensity_g2,
  uint8_t intensity_b2,
  uint8_t mode,
  unsigned long param1,
  unsigned long param2,
  unsigned long param3
  ) : my_pixel_(which_pixel), intensity_r_(intensity_r), intensity_g_(intensity_g), intensity_b_(intensity_b), intensity_r2_(intensity_r2), intensity_g2_(intensity_g2), intensity_b2_(intensity_b2), mode_(mode), param1_(param1), param2_(param2), param3_(param3)
{}

PixelTimer::~PixelTimer(){}

void PixelTimer::set_values(
  uint8_t which_pixel,
  uint8_t intensity_r,
  uint8_t intensity_g,
  uint8_t intensity_b,
  uint8_t intensity_r2,
  uint8_t intensity_g2,
  uint8_t intensity_b2,
  uint8_t mode,
  unsigned long param1, 
  unsigned long param2, 
  unsigned long param3 
  )
{
  my_pixel_= which_pixel; 
  intensity_r_ = intensity_r;
  intensity_g_ = intensity_g;
  intensity_b_ = intensity_b;
  intensity_r2_ = intensity_r2;
  intensity_g2_ = intensity_g2;
  intensity_b2_ = intensity_b2;
  mode_ = mode;
  param1_ = param1;
  param2_ = param2;
  param3_ = param3;
}

void PixelTimer::setPixelColor(uint8_t red, uint8_t green, uint8_t blue)
{
  uint8_t row = my_pixel_ % 8;
  uint8_t col = (my_pixel_ - row) / 8;

  dots[Page_Index][row][col][0] = red; 
  dots[Page_Index][row][col][1] = green; 
  dots[Page_Index][row][col][2] = blue;
}

void PixelTimer::init_on()
{
  PixelTimer::setPixelColor( intensity_r_, intensity_g_, intensity_b_);
}

void PixelTimer::process_time(unsigned long current_time )
{
  // take account of initial delay
  if(param3_ <= current_time)
  {
    current_time -= param3_;
    // switch according to mode
    switch (mode_)
    {
      case 0:
        boxcar(current_time);
        break;
      case 1:
        sine(current_time);
        break;
      default:
        boxcar(current_time);
    }
  }
}

void PixelTimer::boxcar (unsigned long current_time)
{
  unsigned long t_in_current_wave = current_time % (param1_+param2_);

  // check whether we are in on or off phase
  if (t_in_current_wave <= param1_)
  {
      PixelTimer::setPixelColor( intensity_r_, intensity_g_, intensity_b_);
  }
  else
  {
    // check LED status
      PixelTimer::setPixelColor( intensity_r2_, intensity_g2_, intensity_b2_);
  }
}

void PixelTimer::sine( unsigned long current_time)
{
  unsigned long t_in_current_wave = current_time % param1_;
  double sval;

  sval=pow(sin(2.0*PI*(1.0*t_in_current_wave)/(1.0*param1_)),2);
  PixelTimer::setPixelColor( sval*intensity_r_+(1-sval)*intensity_r2_, sval*intensity_g_+(1-sval)*intensity_g2_, sval*intensity_b_+(1-sval)*intensity_b2_);
}

// global variables
PixelTimer ptimers[num_neopixels];
#if USE_RTC
  RTC_DS3231 rtc;
#endif

// check pixels every how many ms?
int delay_time = 25;
long start_sec;
unsigned long start_millis;
#if USE_RTC
  // specify at which interval (in s) the internal clock is synchronized by the RTC
  long time_correct_interval = 10;
  long prev_sec;
#endif

/************************************
 void setup
 ***********************************/
void setup()
{
  _IO_Init();           //Init IO
  _LED_Init();          //Init LED Hardware
  _TC2_Init();          //Init Timer/Count2
 
  // arguments are: pixel ID, red, green, blue, red2, green2, blue2, mode, param1, param2, param3
<<<LED_DEFINITIONS>>>
  
#if USE_RTC
  // set up RTC and reset time
  rtc.begin();
  DateTime now = rtc.now();
  start_sec=now.secondstime();
  prev_sec=0;
#endif

  // record start of millis
  start_millis=millis();
} 

void loop()
{
   unsigned long curr_millis = millis();
#if USE_RTC
   DateTime now = rtc.now();
   long curr_sec=now.secondstime()-start_sec;

   // synchronize millisecond counter to the RTC
   if (curr_sec-time_correct_interval>=prev_sec)
   {
     prev_sec=curr_sec;
     start_millis=curr_millis;
   }
   // only keep the last millis since last update
   curr_millis+=1000*time_correct_interval*floor(curr_sec/time_correct_interval);
   curr_millis-=start_millis;
#endif

  if (Page_Index)      
    Page_Index = 0;
  else      
    Page_Index=1;

   for( int i = 0; i < num_neopixels; i++)
   {
     ptimers[i].process_time(curr_millis);
   }
   
   delay(delay_time);
}'''
    
    # now write .ino file
    try:
      writehandle=open(filename,'w')
      if(writehandle):
        # process the output program
        arduino=re.sub('<<<LED_NUMBER>>>',str(self.LED_NUMBER),arduino,re.DOTALL)
        arduino=re.sub('<<<RTC_SETTING>>>',str(int(self.checkRTC.isChecked())),arduino,re.DOTALL)
        
        # the different LEDs
        outstring=''
        for i in range(self.LED_NUMBER):
          params=[self.leds[i]['qedit'][j].text() for j in range(9)]
          params=[int(j) if (len(j)) else 0 for j in params]
          outstring+='  ptimers[%d].set_values(%d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d);\n'\
            %(i,i,params[0],params[1],params[2],\
            params[4],params[5],params[6],\
            int(self.leds[i]['qradio'][2].isChecked()),params[3],params[7], params[8])
        
        arduino=re.sub('<<<LED_DEFINITIONS>>>',outstring,arduino,re.DOTALL)
        
        # write the output file
        writehandle.write(arduino)
        writehandle.close()
    except:
      pass

#
#
#

class MyForm(QtWidgets.QMainWindow):
  def __init__(self, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    self.ui = Ui_MainWindow()

    # adjust DPI scaling
    self.targetDPI = 96
    actualDPI = QtGui.QPaintDevice.logicalDpiX(self)
    self.DPI_SCALING = 1.0*actualDPI/self.targetDPI
    self.ui = Ui_MainWindow()
    self.ui.setupUi(MainWindow = self, DPI_SCALING = self.DPI_SCALING)

if __name__ == "__main__":
  app = QtWidgets.QApplication(sys.argv)
  # this is the setting you may want to change
  myapp = MyForm()
  myapp.show()
  #app.exec_()
  #sys.exit(0)
  sys.exit(app.exec_())